
/*
 * $Id: distr_test.cc,v 1.2.2.1 1999/05/21 04:19:57 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "Opt.h"
#include "opts.h"
#include "histograms.h"
#include "gadgets.h"

bool ShutdownNow() { Assert(0); return false; }  // XXX: ARGH!

class MyOpts: public OptGrp {
	public:
		MyOpts():
			theHelpOpt(this,      "help",           "list of options"),
			theOutFileName(this,  "out <file>",     "redirect console output", "-"),
			theDistr(this,  "distr <numDistr>",     "distribution to test"),
			theStep(this,  "hist_step <%>",      "bin `size' in histogram output", 0.05/100),
			theCount(this,  "count <int>",      "number of trials", 100000)

			{ theDistr.argType("num"); }

		virtual bool validate() const;

	public:
		HelpOpt theHelpOpt;
		StrOpt theOutFileName;
		DistrOpt theDistr;
		DblOpt theStep;
		IntOpt theCount;
} TheOpts;

bool MyOpts::validate() const {
	if (!theDistr)
		cerr << "must specify the distribution to test" << endl;
	else
		return true;
	return false;
}

static
void configureLogs(int prec) {
	if (TheOpts.theOutFileName && strcmp(TheOpts.theOutFileName, "-"))
		cout = cerr = clog = *new ofstream(TheOpts.theOutFileName);

	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

// set some general stuff and
// propogate cmd line options to corresponding objects
static
void configure() {
	configureLogs(2);
}

// XXX: make this global (also used in lx)
static
void printHist(ostream &os, const Histogram &h) {
	const int totCount = h.stats().count();
	Array<HistogramBin> percs;

	Percentiles(h, percs, TheOpts.theStep); // step%
	if (!percs.count()) {
		os << '?' << endl;
		return;
	}
	//Assert(percs.count() == 100);

	// header
	os << "#bin min max count % acc%" << endl;

	int lastIdx = -1;
	for (int i = 0; i < percs.count(); ++i) {
		const HistogramBin &b = percs[i];
		if (b.idx != lastIdx) {
			os
				<< b.idx
				<< ' ' << b.min
				<< ' ' << b.max
				<< ' ' << b.count
				<< ' ' << setw(6) << Percent(b.count, totCount)
				<< ' ' << setw(6) << Percent(b.accCount, totCount)
				<< endl;
			lastIdx = b.idx;
		}
	}
}

static
void test() {
	const int count = TheOpts.theCount;
	RndDistr *distr = TheOpts.theDistr.condDistr();
	Histogram *hist = new Log2Hist();

	for (int i = 0; i < count; ++i) {
		hist->record((int)distr->trial());
	}

	printHist(cout, *hist);
	delete hist;
}

int main(int argc, char *argv[]) {

	CmdLine cmd(Array<OptGrp*>() << &TheOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate() || !TheOpts.validate())
		return -1;

	configure();

	test();

	return 0;
}
