
/*
 * $Id: httpHdrs.h,v 1.2 1999/04/30 04:47:15 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef HTTP_HEADERS_H
#define HTTP_HEADERS_H

#include <iostream.h>

#include "Array.h"
#include "UniqId.h"
#include "LibInit.h"

class MsgHdrParsTab;

class MsgHdr {
	public:
		typedef bool (MsgHdr::*Parser)(const char *buf, const char *eoh);

	public:
		MsgHdr(const MsgHdrParsTab &aTab);
		virtual ~MsgHdr() {}

		virtual void reset();

		bool keepAlive() const;

		bool parse(const char *buf, Size sz);

	protected:
		static void Configure(MsgHdrParsTab &tab);
		static int AddParser(const String &field, Parser parser, MsgHdrParsTab &tab);

	protected:
		void parseFields();
		virtual bool parseRLine(const char *buf, const char *eol) = 0;

		bool parseContLen(const char *buf, const char *eoh);
		bool parseXXact(const char *buf, const char *eoh);
		bool parseConnection(const char *buf, const char *eoh);

		// these have to be here so we can register them in req
		virtual bool parseXEtag(const char *buf, const char *eoh);
		virtual bool parseIMS(const char *buf, const char *eoh);

		// these have to be here so we can register them in rep
		virtual bool parsePragma(const char *buf, const char *eoh);
		virtual bool parseCControl(const char *buf, const char *eoh);

	public:
		const MsgHdrParsTab &theParsTab;

		Size theHdrSize;
		Size theContSize;
		UniqId theXactId;
		double theHttpVer;
		enum { kaNo, kaDefault, kaYes } theKeepAliveConn;

	protected:
		Array<const char *> theFields; // positions of header fields
		const char *theBufBeg, *theBufEnd;
		const char *theSrchPtr;        // parse buffer iterator
		enum { ssFirst, ssSkip, ssFound } theSrchState;
};

class ReqHdr: public MsgHdr {
	public:
		static void Configure();
		
	public:
		ReqHdr();

		virtual void reset();

		bool foreign() const { return theObjId <= 0 || !theXactId || theETag <= 0; }

	protected:
		virtual bool parseRLine(const char *buf, const char *eol);
		virtual bool parseXEtag(const char *buf, const char *eoh);
		virtual bool parseIMS(const char *buf, const char *eoh);

	protected:
		static MsgHdrParsTab *TheParsTab;

	public:
		Time theIMS;     // If-Modified-Since
		int theObjId;
		int theETag;
};

class RepHdr: public MsgHdr {
	public:
		static void Configure();
		
	public:
		RepHdr();

		virtual void reset();

	protected:
		virtual bool parseRLine(const char *buf, const char *eol);

		virtual bool parsePragma(const char *buf, const char *eoh);
		virtual bool parseCControl(const char *buf, const char *eoh);

	protected:
		static MsgHdrParsTab *TheParsTab;

	public:
		bool isCachable;
};

inline ostream &crlf(ostream &os) { return os.write("\r\n", 2); }

LIB_INITIALIZER(HttpHdrsInit)

#endif
